/*******************************************************

					USB.h
	
	Prototypen fr USB Routinen fr FT245 fr Webserver 
	mit WIZ810MJ
	
	Diese Funktionen sind zum Teil speziell auf den
	AVR Webserver mit Wiznet Modul WIZ810MJ
	abgestimmt, z.B. die Interruptroutine, in der die 
	Daten vom FT245 empfangen werden, damit man in den
	Bootloader des Webservers springen kann.
	
	Eigenschaften:
	
	- Receive Buffer mit einstellbarer Buffersize
	
	- Transmit Buffer mit einstellbarer Buffersize
	
	- Automatische Erkennung, ob der FT245 am USB-Port
	  des PCs angesteckt ist oder nicht (nur unter
	  Verwendung von TXE und RXF)
	  -> Automatische Kompensation der Power-On Pulse
	     und Pulse nach Anstecken am PC an TXE und RXF
		 (genaueres siehe FT245.c)
		 Dies ist ntig, damit das verwendete ber-
		 tragungsprotokoll nicht durcheinanderkommt
	
	- Receive Protokoll:
		1.) START Byte: 0x1B -> Daten von AVR Studio 
						(fr Bootloader reserviert!!!)
						
						!0x1B -> User Daten
		
		2.) Sequence Number
		
		3.) Message Size MSB
		
		4.) Message Size LSB
		
		5.) Datenblock: Block von Message Size Datenbytes
						(je 8 bit Datenbyte)
		
		6.) Checksumme: XOR von allen Bytes

********************************************************/

#ifndef _USB_H_
#define _USB_H_

/********************************************************
*														*
*			Include										*
*														*
*********************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>	// Fr Bootlaoder
#include "FT245.h"
#include "buffer.h"
#include "../BootloaderApplication.h"	// Fr Bootloader

/********************************************************
*														*
*			Defines										*
*														*
*********************************************************/
// Gre des Receive Buffers
#define USB_RX_BUFFER_SIZE 1024
// Gre des Transmit Buffers
#define USB_TX_BUFFER_SIZE 512


/********************************************************
*														*
*			Funktionsprototypen							*
*														*
*********************************************************/


/********************************************************
	Initialisiert die USB Buffer und die USB Interrupts  

Parameter:		NONE

Rckgabewert:	NONE
*********************************************************/

void usbInit(void);


/********************************************************
	Schreibt ein Byte in den Transmit Buffer 

Parameter:		zu schreibendes Byte

Rckgabewert:	1:	Byte erfolgreich zu Transmitbuffer 
					hinzugefgt
				0:	Buffer voll
*********************************************************/

uint8_t usbWriteByte(uint8_t byte);


/********************************************************
	Holt ein Byte vom Receive Buffer  

Parameter:		Pointer auf das Byte, in das neue
				Datenbyte gespeichert werden soll

Rckgabewert:	1:	Byte erfolgreich geholt
				0:	Datenbuffer leer oder Buffergre=0
*********************************************************/

uint8_t usbGetByte(uint8_t* byte);


/********************************************************
	berprft ob neue Daten empfangen wurden oder ob
	Daten geschrieben werden knnen.
	Wenn Daten vorhanden sind, wird die jeweilige
	Interruptroutine ausgefhrt.

Parameter:		NONE

Rckgabewert:	NONE
*********************************************************/

void usbNewData(void);

#endif
